"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PackageFilter = /** @class */ (function () {
    function PackageFilter(config) {
        this.invalidPaths = this.parseBlacklist(config.importBlacklist);
        this.whiteListPaths = this.parseWhitelist(config.importWhitelist);
        this.skipSubFolders = config.skipSubFolders;
    }
    PackageFilter.prototype.parseBlacklist = function (list) {
        return list.split(",");
    };
    PackageFilter.prototype.parseWhitelist = function (list) {
        return list.split(",").filter(function (entry) { return entry.length > 0; });
    };
    PackageFilter.prototype.passesBlacklist = function (importPath) {
        return !this.invalidPaths.includes(importPath);
    };
    PackageFilter.prototype.passesBlackAndWhitelist = function (importPath) {
        return this.passesBlacklist(importPath) && this.passesWhitelist(importPath);
    };
    PackageFilter.prototype.passesWhitelist = function (importPath) {
        return this.whiteListPaths.length === 0 || this.whiteListPaths.includes(importPath);
    };
    PackageFilter.prototype.canOutputSubfoldersOf = function (folder) {
        return this.passesBlackAndWhitelist(folder.importPath);
    };
    PackageFilter.prototype.isImportPathOkForFolder = function (folder) {
        return this.passesBlackAndWhitelist(folder.importPath);
    };
    PackageFilter.prototype.isImportPathOkForFolderWithWhitelist = function (folder) {
        return this.isImportPathOkForFolder(folder) && this.passesWhitelist(folder.importPath);
    };
    PackageFilter.prototype.isImportPathOkForSubFolder = function () {
        // black/white lists do not apply to sub-folders
        return !this.skipSubFolders;
    };
    PackageFilter.prototype.isImportPathOkForEdges = function (importPath) {
        return this.passesBlackAndWhitelist(importPath);
    };
    return PackageFilter;
}());
exports.PackageFilter = PackageFilter;
//# sourceMappingURL=PackageFilter.js.map